@@ -6,17 +6,15 @@ require_once 'Horde/MIME/Viewer/plain.php';
  * The IMP_MIME_Viewer_plain class renders out text/plain MIME parts
  * with URLs made into hyperlinks.
  *
- * $Horde: imp/lib/MIME/Viewer/plain.php,v 1.58.8.18 2007/01/12 14:40:51 jan Exp $
+ * $Horde: imp/lib/MIME/Viewer/plain.php,v 1.58.8.24 2008/08/07 16:32:24 slusarz Exp $
  *
- * Copyright 1999-2007 Anil Madhavapeddy <anil@recoil.org>
- * Copyright 2002-2007 Michael Slusarz <slusarz@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Anil Madhavapeddy <anil@recoil.org>
  * @author  Michael Slusarz <slusarz@horde.org>
- * @since   IMP 4.0
  * @package Horde_MIME_Viewer
  */
 class IMP_MIME_Viewer_plain extends MIME_Viewer_plain {
@@ -43,7 +41,7 @@ class IMP_MIME_Viewer_plain extends MIME_Viewer_plain {
         if ($contents->viewAsAttachment()) {
             // Check for 'flowed' text data.
             if ($flowed) {
-                $text = $this->_formatFlowed($text, 90, null, $this->mime_part->getContentTypeParameter('delsp'));
+                $text = $this->_formatFlowed($text, null, 0, $this->mime_part->getContentTypeParameter('delsp'));
             }
             return $text;
         }
@@ -63,7 +61,7 @@ class IMP_MIME_Viewer_plain extends MIME_Viewer_plain {
             $prefs->getValue('pgp_scan_body') &&
             preg_match('/-----BEGIN PGP ([^-]+)-----/', $text)) {
             require_once IMP_BASE . '/lib/Crypt/PGP.php';
-            $imp_pgp = &new IMP_PGP();
+            $imp_pgp = new IMP_PGP();
             if (($out = $imp_pgp->parseMessageOutput($this->mime_part, $contents))) {
                 return $out;
             }
@@ -71,36 +69,47 @@ class IMP_MIME_Viewer_plain extends MIME_Viewer_plain {
 
         // If requested, scan the message for UUencoded data.
         if ($this->getConfigParam('uuencode')) {
+            // Don't want to use convert_uudecode() here as there may be
+            // multiple files residing in the text.
             require_once 'Mail/mimeDecode.php';
             $files = &Mail_mimeDecode::uudecode($text);
         }
 
         // Check for 'flowed' text data.
         if ($flowed) {
-            $text = $this->_formatFlowed($text, 90, (IMP::printMode()) ? 0 : null, $this->mime_part->getContentTypeParameter('delsp'));
+            $text = $this->_formatFlowed($text, null, 0, $this->mime_part->getContentTypeParameter('delsp'));
+        } else {
+            /* A "From" located at the beginning of a line in the body text
+             * will be escaped with a '>' by the IMAP server.  Remove this
+             * escape character or else the line will display as being
+             * quoted. Flowed conversion would have already taken care of this
+             * for us. */
+            $text = preg_replace('/(\n+)> ?From(\s+)/', "$1From$2", $text);
         }
 
         // Build filter stack. Starts with HTML markup and tab expansion.
         require_once 'Horde/Text/Filter.php';
         $filters = array('text2html', 'tabs2spaces');
-        $filter_params = array(array('parselevel' => TEXT_HTML_MICRO,
-                                     'charset' => $this->mime_part->getCharset()),
-                               array());
-
-        /* A "From" located at the beginning of a line in the body text will
-         * be escaped with a '>' by the IMAP server.  Remove this escape
-         * character or else the line will display as being quoted. Flowed
-         * conversion would have already taken care of this for us. */
-        if (!$flowed) {
-            $text = preg_replace('/(\n+)> ?From(\s+)/', "$1From$2", $text);
-        }
+        $filter_params = array(
+            array('parselevel' => TEXT_HTML_MICRO,
+                  'charset' => $this->mime_part->getCharset()),
+            array());
 
         // Highlight quoted parts of an email.
         if ($prefs->getValue('highlight_text')) {
             $filters[] = 'highlightquotes';
-            $hideBlocks = ($prefs->getValue('show_quoteblocks') == 'hidden') ||
-                ($prefs->getValue('show_quoteblocks') == 'thread' &&
+
+            $show = $prefs->getValue('show_quoteblocks');
+            $hideBlocks = ($show == 'hidden') ||
+                ($show == 'thread' &&
                  basename(Horde::selfUrl()) == 'thread.php');
+            if (!$hideBlocks &&
+                in_array($show, array('list', 'listthread')) &&
+                is_a($contents, 'IMP_Contents')) {
+                $header = $contents->getHeaderOb();
+                $list_info = $header->getListInformation();
+                $hideBlocks = $list_info['exists'];
+            }
             $filter_params[] = array('hideBlocks' => $hideBlocks);
         }
 
@@ -129,7 +138,7 @@ class IMP_MIME_Viewer_plain extends MIME_Viewer_plain {
         // Wordwrap.
         $text = str_replace('  ', ' &nbsp;', $text);
         $text = str_replace("\n ", "\n&nbsp;", $text);
-        if ($text{0} == ' ') {
+        if (!strncmp($text, ' ', 1)) {
             $text = '&nbsp;' . substr($text, 1);
         }
         $text = '<div class="fixed leftAlign">' . "\n" . $text . '</div>';
@@ -137,12 +146,12 @@ class IMP_MIME_Viewer_plain extends MIME_Viewer_plain {
         // Replace UUencoded data with links now.
         if ($this->getConfigParam('uuencode') && !empty($files)) {
             foreach ($files as $file) {
-                $uupart = &new MIME_Part();
+                $uupart = new MIME_Part();
                 $uupart->setContents($file['filedata']);
                 $uupart->setName(strip_tags($file['filename']));
 
                 $uumessage = &MIME_Message::convertMIMEPart($uupart);
-                $mc = &new MIME_Contents($uumessage, array('download' => 'download_attach', 'view' => 'view_attach'), array(&$contents));
+                $mc = new MIME_Contents($uumessage, array('download' => 'download_attach', 'view' => 'view_attach'), array(&$contents));
                 $mc->buildMessage();
 
                 $text = preg_replace("/begin ([0-7]{3}) (.+)\r?\n(.+)\r?\nend/Us", '<table>' . $mc->getMessage(true) . '</table>', $text, 1);
