@@ -5,12 +5,11 @@ define('IMP_SEARCH_HEADER', 1);
 define('IMP_SEARCH_BODY', 2);
 define('IMP_SEARCH_DATE', 3);
 define('IMP_SEARCH_TEXT', 4);
+define('IMP_SEARCH_SIZE', 5);
 
 /* Defines used to identify the flag input. */
-define('IMP_SEARCH_FLAG_SEEN', 1);
-define('IMP_SEARCH_FLAG_ANSWERED', 2);
-define('IMP_SEARCH_FLAG_FLAGGED', 3);
-define('IMP_SEARCH_FLAG_DELETED', 4);
+define('IMP_SEARCH_FLAG_NOT', 0);
+define('IMP_SEARCH_FLAG_HAS', 1);
 
 /* Defines used to identify whether to show unsubscribed folders. */
 define('IMP_SEARCH_SHOW_UNSUBSCRIBED', 0);
@@ -41,15 +40,14 @@ define('IMP_SEARCH_SHOW_SUBSCRIBED_ONLY', 1);
  *     'vinbox_id' => string (The Virtual Inbox query ID)
  * );
  *
- * $Horde: imp/lib/Search.php,v 1.37.10.37 2007/01/02 13:54:56 jan Exp $
+ * $Horde: imp/lib/Search.php,v 1.37.10.45 2008/08/22 16:35:44 jan Exp $
  *
- * Copyright 2002-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @since   IMP 4.0
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @package IMP
  */
 class IMP_Search {
@@ -61,13 +59,6 @@ class IMP_Search {
      */
     var $_id = null;
 
-    /**
-     * The IMP_Tree:: object to update.
-     *
-     * @var object
-     */
-    var $_imptree;
-
     /**
      * Save Virtual Folder information when adding entries?
      *
@@ -92,18 +83,22 @@ class IMP_Search {
 
     /**
      * Set up IMP_Search variables for the current session.
+     *
+     * @param boolean $no_vf  Don't readd the Virtual Folders.
      */
-    function sessionSetup()
+    function sessionSetup($no_vf = false)
     {
         if (!isset($_SESSION['imp']['search'])) {
             $_SESSION['imp']['search'] = array('q' => array());
         }
-        foreach ($this->_getVFolderList() as $key => $val) {
-            if (!empty($val['vfolder']) &&
-                !$this->isVTrashFolder($key) &&
-                !$this->isVINBOXFolder($key)) {
-                $_SESSION['imp']['search']['q'][$key] = $val;
-                $this->_updateIMPTree('add', $key, $val['label']);
+        if (!$no_vf) {
+            foreach ($this->_getVFolderList() as $key => $val) {
+                if (!empty($val['vfolder']) &&
+                    !$this->isVTrashFolder($key) &&
+                    !$this->isVINBOXFolder($key)) {
+                    $this->_updateIMPTree('add', $key, $val['label']);
+                    $_SESSION['imp']['search']['q'][$key] = $val;
+                }
             }
         }
         $this->createVINBOXFolder();
@@ -123,46 +118,34 @@ class IMP_Search {
      */
     function runSearch(&$ob, $id = null)
     {
-        $id = (is_null($id)) ? $this->_id : $this->_strip($id);
+        $id = $this->_strip($id);
         $mbox = '';
         $sorted = array();
-        $use_pop3 = ($_SESSION['imp']['base_protocol'] == 'pop3');
 
         if (empty($_SESSION['imp']['search']['q'][$id])) {
             return $sorted;
         }
         $search = &$_SESSION['imp']['search']['q'][$id];
 
-        $charset = NLS::getCharset();
-        $search_params = array('pop3' => $use_pop3, 'charset' => $charset);
-
-        /* Check if the IMAP server supports searches in the current
-         * charset. */
-        if (empty($_SESSION['imp']['imap_server']['search_charset'][$charset])) {
-            $search_params['no_imap_charset'] = true;
-        }
-
-        require_once IMP_BASE . '/lib/IMAP/Search.php';
-        $imap_search = &IMP_IMAP_Search::singleton($search_params);
+        $imap_search = &$this->_getIMAPSearch();
 
         /* Prepare the search query. */
         if (!empty($ob)) {
             $old_query = unserialize($search['query']);
-            $query = &new IMP_IMAP_Search_Query();
+            $query = new IMP_IMAP_Search_Query();
             $query->imapAnd(array($ob, $old_query));
         } else {
             $query = unserialize($search['query']);
         }
 
         /* How do we want to sort results? */
-        $sortby = $GLOBALS['prefs']->getValue('sortby');
-        if ($sortby == SORTTHREAD) {
-            $sortby = SORTDATE;
+        $sortpref = IMP::getSort();
+        if ($sortpref['by'] == SORTTHREAD) {
+            $sortpref['by'] = SORTDATE;
         }
-        $sortdir = $GLOBALS['prefs']->getValue('sortdir');
 
         foreach ($search['folders'] as $val) {
-            $results = $imap_search->searchSortMailbox($query, $_SESSION['imp']['stream'], $val, $sortby, $sortdir);
+            $results = $imap_search->searchSortMailbox($query, null, $val, $sortpref['by'], $sortpref['dir']);
 
             if (is_array($results)) {
                 foreach ($results as $val2) {
@@ -174,6 +157,23 @@ class IMP_Search {
         return $sorted;
     }
 
+    /**
+     * Run a search query not stored in the current session.  Allows custom
+     * queries with custom sorts to be used without affecting cached
+     * mailboxes.
+     *
+     * @since IMP 4.2
+     *
+     * @param IMAP_Search_Query $ob  The search query.
+     *
+     * @return array  The sorted list.
+     */
+    function runSearchQuery($ob, $mailbox, $sortby, $sortdir)
+    {
+        $imap_search = &$this->_getIMAPSearch();
+        return $imap_search->searchSortMailbox($ob, null, $mailbox, $sortby, $sortdir);
+    }
+
     /**
      * Creates the IMAP search query in the IMP session.
      *
@@ -211,7 +211,7 @@ class IMP_Search {
      */
     function deleteSearchQuery($id = null)
     {
-        $id = (is_null($id)) ? $this->_id : $this->_strip($id);
+        $id = $this->_strip($id);
         $is_vfolder = !empty($_SESSION['imp']['search']['q'][$id]['vfolder']);
         unset($_SESSION['imp']['search']['q'][$id]);
 
@@ -233,9 +233,9 @@ class IMP_Search {
      */
     function retrieveUIQuery($id = null)
     {
-        $id = (is_null($id)) ? $this->_id : $this->_strip($id);
+        $id = $this->_strip($id);
         return (isset($_SESSION['imp']['search']['q'][$id]['uiinfo']))
-            ? $GLOBALS['imp']['search']['q'][$id]['uiinfo']
+            ? $_SESSION['imp']['search']['q'][$id]['uiinfo']
             : array();
     }
 
@@ -249,9 +249,9 @@ class IMP_Search {
      */
     function getLabel($id = null)
     {
-        $id = (is_null($id)) ? $this->_id : $this->_strip($id);
+        $id = $this->_strip($id);
         return (isset($_SESSION['imp']['search']['q'][$id]['label']))
-            ? $GLOBALS['imp']['search']['q'][$id]['label']
+            ? $_SESSION['imp']['search']['q'][$id]['label']
             : '';
     }
 
@@ -269,30 +269,44 @@ class IMP_Search {
             return array();
         }
 
-        $vfolder = @unserialize($vfolder);
+        $old_error = error_reporting(0);
+        $vfolder = unserialize($vfolder);
+        error_reporting($old_error);
+
         if (!is_array($vfolder)) {
-            return array();
+            $vfolder = array();
         }
 
-        $elt = reset($vfolder);
-        if (!isset($elt['ob'])) {
-            // Already in 4.1+ format.
-            return $vfolder;
+        // BC: Convert old (IMP < 4.2.1) style w/separate flag entry to new
+        // style where flags are part of the fields to query.
+        if (!empty($vfolder)) {
+            $entry = reset($vfolder);
+            if (isset($entry['uiinfo']['flag'])) {
+                $lookup = array(
+                    1 => 'seen',
+                    2 => 'answered',
+                    3 => 'flagged',
+                    4 => 'deleted'
+                );
+                while (list($k, $v) = each($vfolder)) {
+                    $u = &$v['uiinfo'];
+                    if (!empty($u['flag'])) {
+                        foreach ($u['flag'] as $key => $val) {
+                            if (($val == 0) || ($val == 1)) {
+                                $u['field'][] = (($val == 1) ? 'un' : '') . $lookup[$key];
+                                ++$u['field_end'];
+                            }
+                        }
+                    }
+                    unset($u['flag']);
+                    $v['query'] = serialize($this->createQuery($u));
+                    $vfolder[$k] = $v;
+                }
+                $this->_saveVFolderList($vfolder);
+            }
         }
 
-        // Convert from 4.0 format to 4.1+ format, if necessary
-        $convert = array();
-        foreach ($vfolder as $key => $val) {
-            $convert[$key] = array(
-                'query' => $val['ob'],
-                'folders' => $val['search']['folders'],
-                'uiinfo' => $val['search'],
-                'label' => $val['search']['vfolder_label'],
-                'vfolder' => true
-            );
-            unset($convert[$key]['search']['folders'], $convert[$key]['search']['vfolder_label']);
-        }
-        return $convert;
+        return $vfolder;
     }
 
     /**
@@ -361,7 +375,7 @@ class IMP_Search {
         array_unshift($flist, 'INBOX');
 
         require_once IMP_BASE . '/lib/IMAP/Search.php';
-        $query = &new IMP_IMAP_Search_query();
+        $query = new IMP_IMAP_Search_Query();
         $query->deleted(true);
         $label = _("Virtual Trash");
 
@@ -387,7 +401,7 @@ class IMP_Search {
      */
     function isVTrashFolder($id = null)
     {
-        $id = (is_null($id)) ? $this->_id : $this->_strip($id);
+        $id = $this->_strip($id);
         $vtrash_id = $GLOBALS['prefs']->getValue('vtrash_id');
         return (!empty($vtrash_id) && ($id == $vtrash_id));
     }
@@ -401,7 +415,7 @@ class IMP_Search {
         require_once IMP_BASE . '/lib/IMAP/Tree.php';
         $imptree = &IMP_Tree::singleton();
 
-        /* Delete the current Virtual Trash folder, if it exists. */
+        /* Delete the current Virtual Inbox folder, if it exists. */
         $vinbox_id = $GLOBALS['prefs']->getValue('vinbox_id');
         if (!empty($vinbox_id)) {
             $this->deleteSearchQuery($vinbox_id);
@@ -412,17 +426,11 @@ class IMP_Search {
         }
 
         /* Create Virtual INBOX with nav_poll list. Filter out any nav_poll
-         * entries that don't exist. */
-        $flist = array_values(array_intersect(array_keys($imptree->getPollList()), $imptree->folderList()));
-
-        /* Sort Virtual INBOX list. */
-        require_once IMP_BASE . '/lib/IMAP/Sort.php';
-        $ns_new = IMP::getNamespace();
-        $imap_sort = new IMP_IMAP_Sort($ns_new['delimiter']);
-        $imap_sort->sortMailboxes($flist);
+         * entries that don't exist. Sort the list also. */
+        $flist = $imptree->getPollList(true, true);
 
         require_once IMP_BASE . '/lib/IMAP/Search.php';
-        $query = &new IMP_IMAP_Search_query();
+        $query = new IMP_IMAP_Search_Query();
         $query->seen(false);
         $query->deleted(false);
         $label = _("Virtual INBOX");
@@ -449,7 +457,7 @@ class IMP_Search {
      */
     function isVINBOXFolder($id = null)
     {
-        $id = (is_null($id)) ? $this->_id : $this->_strip($id);
+        $id = $this->_strip($id);
         $vinbox_id = $GLOBALS['prefs']->getValue('vinbox_id');
         return (!empty($vinbox_id) && ($id == $vinbox_id));
     }
@@ -465,7 +473,7 @@ class IMP_Search {
      */
     function isEditableVFolder($id = null)
     {
-        $id = (is_null($id)) ? $this->_id : $this->_strip($id);
+        $id = $this->_strip($id);
         return ($this->isVFolder($id) && !$this->isVTrashFolder($id) && !$this->isVINBOXFolder($id));
     }
 
@@ -505,21 +513,97 @@ class IMP_Search {
      */
     function getSearchFolders($id = null)
     {
-        $id = (is_null($id)) ? $this->_id : $this->_strip($id);
+        $id = $this->_strip($id);
         return (isset($_SESSION['imp']['search']['q'][$id]['folders'])) ? $_SESSION['imp']['search']['q'][$id]['folders'] : array();
     }
 
+    /**
+     * Return a list of search queries valid only for the current session
+     * (i.e. no virtual folders).
+     *
+     * @return array  Keys are the search ids, values are a textual
+     *                description of the search.
+     */
+    function getSearchQueries()
+    {
+        $retarray = array();
+
+        foreach ($_SESSION['imp']['search']['q'] as $key => $val) {
+            if (!$this->isVFolder($key) &&
+                ($text = $this->searchQueryText($key))) {
+                $retarray[$key] = $text;
+            }
+        }
+
+        return array_reverse($retarray, true);
+    }
+
+    /**
+     * Return search query text representation for a given search ID.
+     *
+     * @since IMP 4.2
+     *
+     * @param string $id  The search query id to use (by default, will use
+     *                    the current ID set in the object).
+     *
+     * @return array  The textual description of the search.
+     */
+    function searchQueryText($id = null)
+    {
+        $id = $this->_strip($id);
+        if (empty($_SESSION['imp']['search']['q'][$id])) {
+            return '';
+        } elseif ($this->isVINBOXFolder($id) || $this->isVTrashFolder($id)) {
+            return $_SESSION['imp']['search']['q'][$id]['label'];
+        } elseif (empty($_SESSION['imp']['search']['q'][$id]['uiinfo'])) {
+            unset($_SESSION['imp']['search']['q'][$id]);
+            return '';
+        }
+
+        $flagfields = $this->flagFields();
+        $searchfields = $this->searchFields();
+        $val = $_SESSION['imp']['search']['q'][$id];
+
+        $text = '';
+        if (!empty($val['uiinfo']['field'])) {
+            $text = _("Search") . ' ';
+            $text_array = array();
+            foreach ($val['uiinfo']['field'] as $key2 => $val2) {
+                if (isset($flagfields[$val2])) {
+                    $text_array[] = $flagfields[$val2]['label'];
+                } else {
+                    switch ($searchfields[$val2]['type']) {
+                    case IMP_SEARCH_DATE:
+                        $text_array[] = sprintf("%s '%s'", $searchfields[$val2]['label'], strftime("%x", mktime(0, 0, 0, $val['uiinfo']['date'][$key2]['month'], $val['uiinfo']['date'][$key2]['day'], $val['uiinfo']['date'][$key2]['year'])));
+                        break;
+
+                    case IMP_SEARCH_SIZE:
+                        $text_array[] = $searchfields[$val2]['label'] . ' ' . ($val['uiinfo']['text'][$key2] / 1024);
+                        break;
+
+                    default:
+                        $text_array[] = sprintf("%s for '%s'", $searchfields[$val2]['label'], ((!empty($val['uiinfo']['text_not'][$key2])) ? _("not") . ' ' : '') . $val['uiinfo']['text'][$key2]);
+                        break;
+                    }
+                }
+            }
+            $text .= implode(' ' . (($val['uiinfo']['match'] == 'and') ? _("and") : _("or")) . ' ', $text_array);
+        }
+
+        return $text . ' ' . _("in") . ' ' . implode(', ', $val['uiinfo']['folders']);
+    }
+
     /**
      * Returns a link to edit a given search query.
      *
      * @param string $id  The search query id to use (by default, will use
      *                    the current ID set in the object).
      *
-     * @rerturn string  The URL to the search page.
+     * @return string  The URL to the search page.
      */
     function editURL($id = null)
     {
-        $id = (is_null($id)) ? $this->_id : $this->_strip($id);
+        $id = $this->_strip($id);
         return Util::addParameter(Horde::applicationUrl('search.php'), array('edit_query' => $id));
     }
 
@@ -533,8 +617,12 @@ class IMP_Search {
      */
     function deleteURL($id = null)
     {
-        $id = (is_null($id)) ? $this->_id : $this->_strip($id);
-        return Util::addParameter(Horde::applicationUrl('folders.php'), array('actionID' => 'delete_search_query', 'queryid' => $id));
+        $id = $this->_strip($id);
+        return Util::addParameter(Horde::applicationUrl('folders.php'),
+                                  array('actionID' => 'delete_search_query',
+                                        'folders_token' => IMP::getRequestToken('imp.folders'),
+                                        'queryid' => $id,
+                                  ));
     }
 
     /**
@@ -547,7 +635,7 @@ class IMP_Search {
      */
     function isSearchMbox($id = null)
     {
-        return (is_null($id)) ? !empty($this->_id) : isset($_SESSION['imp']['search']['q'][$this->_strip($id)]);
+        return ($id === null) ? !empty($this->_id) : isset($_SESSION['imp']['search']['q'][$this->_strip($id)]);
     }
 
     /**
@@ -560,7 +648,7 @@ class IMP_Search {
      */
     function isVFolder($id = null)
     {
-        $id = (is_null($id)) ? $this->_id : $this->_strip($id);
+        $id = $this->_strip($id);
         return (!empty($_SESSION['imp']['search']['q'][$id]['vfolder']));
     }
 
@@ -572,7 +660,7 @@ class IMP_Search {
      */
     function searchMboxID()
     {
-        return (!is_null($this->_id)) ? $this->_id : false;
+        return ($this->_id !== null) ? $this->_id : false;
     }
 
     /**
@@ -587,7 +675,7 @@ class IMP_Search {
      */
     function _strip($id)
     {
-        return (strpos($id, IMP_SEARCH_MBOX) === 0) ? substr($id, strlen(IMP_SEARCH_MBOX)) : $id;
+        return ($id === null) ? $this->_id : ((strpos($id, IMP_SEARCH_MBOX) === 0) ? substr($id, strlen(IMP_SEARCH_MBOX)) : $id);
     }
 
     /**
@@ -616,44 +704,119 @@ class IMP_Search {
         return array(
             'from' => array(
                 'label' => _("From"),
-                'type' => IMP_SEARCH_HEADER
+                'type' => IMP_SEARCH_HEADER,
+                'not' => true
             ),
             'to' => array(
                 'label' => _("To"),
-                'type' => IMP_SEARCH_HEADER
+                'type' => IMP_SEARCH_HEADER,
+                'not' => true
             ),
             'cc' => array(
                 'label' => _("Cc"),
-                'type' => IMP_SEARCH_HEADER
+                'type' => IMP_SEARCH_HEADER,
+                'not' => true
             ),
             'bcc' => array(
                 'label' => _("Bcc"),
-                'type' => IMP_SEARCH_HEADER
+                'type' => IMP_SEARCH_HEADER,
+                'not' => true
             ),
             'subject' => array(
                 'label' => _("Subject"),
-                'type' => IMP_SEARCH_HEADER
+                'type' => IMP_SEARCH_HEADER,
+                'not' => true
             ),
             'body' => array(
                'label' => _("Body"),
-               'type' => IMP_SEARCH_BODY
+               'type' => IMP_SEARCH_BODY,
+                'not' => true
             ),
             'text' => array(
                'label' => _("Entire Message"),
-               'type' => IMP_SEARCH_TEXT
+               'type' => IMP_SEARCH_TEXT,
+                'not' => true
             ),
-            'received_on' => array(
-                'label' => _("Received On"),
-                'type' => IMP_SEARCH_DATE
+            'date_on' => array(
+                'label' => _("Date ="),
+                'type' => IMP_SEARCH_DATE,
+                'not' => true
             ),
-            'received_until' => array(
-                'label' => _("Received Until"),
-                'type' => IMP_SEARCH_DATE
+            'date_until' => array(
+                'label' => _("Date <"),
+                'type' => IMP_SEARCH_DATE,
+                'not' => true
+            ),
+            'date_since' => array(
+                'label' => _("Date >="),
+                'type' => IMP_SEARCH_DATE,
+                'not' => true
+            ),
+            // Displayed in KB, but stored internally in bytes
+            'size_smaller' => array(
+                'label' => _("Size (KB) <"),
+                'type' => IMP_SEARCH_SIZE,
+                'not' => false
+            ),
+            // Displayed in KB, but stored internally in bytes
+            'size_larger' => array(
+                'label' => _("Size (KB) >"),
+                'type' => IMP_SEARCH_SIZE,
+                'not' => false
+            ),
+        );
+    }
+
+    /**
+     * Return the base flag fields.
+     *
+     * @since IMP 4.2.1
+     *
+     * @return array  The base flag fields.
+     */
+    function flagFields()
+    {
+        return array(
+            'seen' => array(
+                'flag' => 'seen',
+                'label' => _("Seen messages"),
+                'type' => IMP_SEARCH_FLAG_HAS
+            ),
+            'unseen' => array(
+                'flag' => 'seen',
+                'label' => _("Unseen messages"),
+                'type' => IMP_SEARCH_FLAG_NOT
+            ),
+            'answered' => array(
+                'flag' => 'answered',
+                'label' => _("Answered messages"),
+                'type' => IMP_SEARCH_FLAG_HAS
+            ),
+            'unanswered' => array(
+                'flag' => 'answered',
+                'label' => _("Unanswered messages"),
+                'type' => IMP_SEARCH_FLAG_NOT
+            ),
+            'flagged' => array(
+                'flag' => 'flagged',
+                'label' => _("Flagged messages"),
+                'type' => IMP_SEARCH_FLAG_HAS
+            ),
+            'unflagged' => array(
+                'flag' => 'flagged',
+                'label' => _("Unflagged messages"),
+                'type' => IMP_SEARCH_FLAG_NOT
+            ),
+            'deleted' => array(
+                'flag' => 'deleted',
+                'label' => _("Deleted messages"),
+                'type' => IMP_SEARCH_FLAG_HAS
+            ),
+            'undeleted' => array(
+                'flag' => 'deleted',
+                'label' => _("Undeleted messages"),
+                'type' => IMP_SEARCH_FLAG_NOT
             ),
-            'received_since' => array(
-                'label' => _("Received Since"),
-                'type' => IMP_SEARCH_DATE
-            )
         );
     }
 
@@ -669,20 +832,124 @@ class IMP_Search {
      */
     function _updateIMPTree($action, $id, $label = null)
     {
-        if (empty($this->_imptree)) {
-            require_once IMP_BASE . '/lib/IMAP/Tree.php';
-            $this->_imptree = &IMP_Tree::singleton();
-        }
+        require_once IMP_BASE . '/lib/IMAP/Tree.php';
+        $imptree = &IMP_Tree::singleton();
 
         switch ($action) {
         case 'delete':
-            $this->_imptree->delete($id);
+            $imptree->delete($id);
             break;
 
         case 'add':
-            $this->_imptree->insertVFolders(array($id => $label));
+            $imptree->insertVFolders(array($id => $label));
             break;
         }
     }
 
+    /**
+     * Return an IMAP_Search object.
+     *
+     * @access private
+     *
+     * @return IMAP_Search  The IMAP_Search object.
+     */
+    function &_getIMAPSearch()
+    {
+        $charset = NLS::getCharset();
+        $search_params = array('pop3' => ($_SESSION['imp']['base_protocol'] == 'pop3'), 'charset' => $charset);
+
+        /* Check if the IMAP server supports searches in the current
+         * charset. */
+        if (empty($_SESSION['imp']['imap_server']['search_charset'][$charset])) {
+            $search_params['no_imap_charset'] = true;
+        }
+
+        require_once IMP_BASE . '/lib/IMAP/Search.php';
+        $imap_search = &IMP_IMAP_Search::singleton($search_params);
+
+        return $imap_search;
+    }
+
+    /**
+     * Creates a search query.
+     *
+     * @since IMP 4.2.1
+     *
+     * @param array $uiinfo  An uiinfo array (see imp/search.php).
+     *
+     * @return IMP_IMAP_Search_Query  A search object.
+     */
+    function createQuery($search)
+    {
+        require_once IMP_BASE . '/lib/IMAP/Search.php';
+        $query = new IMP_IMAP_Search_Query();
+
+        $search_array = array();
+        $search_fields = $this->searchFields();
+        $flag_fields = $this->flagFields();
+
+        foreach ($search['field'] as $key => $val) {
+            $ob = new IMP_IMAP_Search_Query();
+
+            if (isset($flag_fields[$val])) {
+                $ob->$flag_fields[$val]['flag']((bool)$flag_fields[$val]['type']);
+                $search_array[] = $ob;
+            } else {
+                switch ($search_fields[$val]['type']) {
+                case IMP_SEARCH_HEADER:
+                    if (!empty($search['text'][$key])) {
+                        $ob->header($val, $search['text'][$key], $search['text_not'][$key]);
+                        $search_array[] = $ob;
+                    }
+                    break;
+
+                case IMP_SEARCH_BODY:
+                    if (!empty($search['text'][$key])) {
+                        $ob->body($search['text'][$key], $search['text_not'][$key]);
+                        $search_array[] = $ob;
+                    }
+                    break;
+
+                case IMP_SEARCH_TEXT:
+                    if (!empty($search['text'][$key])) {
+                        $ob->text($search['text'][$key], $search['text_not'][$key]);
+                        $search_array[] = $ob;
+                    }
+                    break;
+
+                case IMP_SEARCH_DATE:
+                    if (!empty($search['date'][$key]['day']) &&
+                        !empty($search['date'][$key]['month']) &&
+                        !empty($search['date'][$key]['year'])) {
+                        if ($val == 'date_on') {
+                            $ob->on($search['date'][$key]['day'], $search['date'][$key]['month'], $search['date'][$key]['year']);
+                        } elseif ($val == 'date_until') {
+                            $ob->before($search['date'][$key]['day'], $search['date'][$key]['month'], $search['date'][$key]['year']);
+                        } elseif ($val == 'date_since') {
+                            $ob->since($search['date'][$key]['day'], $search['date'][$key]['month'], $search['date'][$key]['year']);
+                        }
+                        $search_array[] = $ob;
+                    }
+                    break;
+
+                case IMP_SEARCH_SIZE:
+                    if (!empty($search['text'][$key])) {
+                        $ob->size(intval($search['text'][$key]), ($val == 'size_larger') ? '>' : '<');
+                        $search_array[] = $ob;
+                    }
+                    break;
+                }
+            }
+        }
+
+        /* Search match. */
+        if ($search['match'] == 'and') {
+            $query->imapAnd($search_array);
+        } elseif ($search['match'] == 'or') {
+            $query->imapOr($search_array);
+        }
+
+        return $query;
+    }
+
 }
