@@ -1,13 +1,14 @@
 <?php
 /**
- * $Horde: imp/test.php,v 1.33.6.12 2006/06/08 05:32:54 slusarz Exp $
+ * $Horde: imp/test.php,v 1.33.6.18 2008/08/25 05:57:04 slusarz Exp $
  *
- * Copyright 2002-2006 Brent J. Nordquist <bjn@horde.org>
- * Copyright 1999-2006 Charles J. Hagenbuch <chuck@horde.org>
- * Copyright 1999-2006 Jon Parise <jon@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL).  If you
+ * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ *
+ * @author Brent J. Nordquist <bjn@horde.org>
+ * @author Chuck Hagenbuch <chuck@horde.org>
  */
 
 /* Include Horde's core.php file. */
@@ -17,17 +18,17 @@ include_once '../lib/core.php';
  * package, in core.php. If String:: isn't defined, then we're not
  * finding some critical libraries. */
 if (!class_exists('String')) {
-    echo '<br /><span style="color: red; font-size: 18px; font-weight: bold;">The Horde_Util package was not found. If PHP\'s error_reporting setting is high enough, there should be error messages printed above that may help you in debugging the problem. If you are simply missing these files, then you need to get the <a href="http://cvs.horde.org/cvs.php/framework">framework</a> module from <a href="http://horde.org/source/">Horde CVS</a>, and install the packages in it with the install-packages.php script.</span>';
+    echo '<br /><h2 style="color:red">The Horde_Util package was not found. If PHP\'s error_reporting setting is high enough and display_errors is on, there should be error messages printed above that may help you in debugging the problem. If you are simply missing these files, then you need to get the <a href="http://cvs.horde.org/cvs.php/framework">framework</a> module from <a href="http://www.horde.org/source/">Horde CVS</a>, and install the packages in it with the install-packages.php script.</h2>';
     exit;
 }
 
 /* Initialize the Horde_Test:: class. */
-if (!(@is_readable('../lib/Test.php'))) {
+if (!is_readable('../lib/Test.php')) {
     echo 'ERROR: You must install Horde before running this script.';
     exit;
 }
 require_once '../lib/Test.php';
-$horde_test = &new Horde_Test;
+$horde_test = new Horde_Test;
 
 /* IMP version. */
 $module = 'IMP';
@@ -39,6 +40,10 @@ require TEST_TEMPLATES . 'version.inc';
 
 /* Display versions of other Horde applications. */
 $app_list = array(
+    'dimp' => array(
+        'error' => 'DIMP provides an alternate display view using JavaScript.',
+        'version' => '1.0'
+    ),
     'gollem' => array(
         'error' => 'Gollem provides access to local VFS filesystems to attach files.',
         'version' => '1.0'
@@ -47,6 +52,10 @@ $app_list = array(
         'error' => 'Ingo provides basic mail filtering capabilities to IMP.',
         'version' => '1.0'
     ),
+    'mimp' => array(
+        'error' => 'MIMP provides an alternate display view suitable for mobile browsers or very slow connections.',
+        'version' => '1.0'
+    ),
     'nag' => array(
         'error' => 'Nag allows tasks to be directly created from e-mail data.',
         'version' => '2.0'
@@ -71,15 +80,25 @@ require TEST_TEMPLATES . 'php_version.inc';
 
 /* PHP modules. */
 $module_list = array(
+    'idn' => array(
+        'descrip' => 'Internationalized Domain Names Support',
+        'error' => 'IMP requires the idn module (installed via PECL) in order to handle Internationalized Domain Names.',
+        'fatal' => false
+    ),
     'imap' => array(
         'descrip' => 'IMAP Support',
-        'error' => 'IMP requires the imap module to interact with the mail server.  It is required even if you only use pop3 access.',
+        'error' => 'IMP requires the imap module to interact with the mail server.  It is required for either IMAP or POP3 access.',
         'fatal' => true
     ),
     'openssl' => array(
         'descrip' => 'OpenSSL Support',
         'error' => 'The openssl module is required to use S/MIME in IMP. Compile PHP with <code>--with-openssl</code> to activate.',
         'fatal' => false
+    ),
+    'tidy' => array(
+        'descrip' => 'Tidy support',
+        'error' => 'If the tidy PHP extension is available, IMP can use it to sanitize the output of HTML messages before displaying to the user, and to clean outgoing HTML messages created in the HTML composition mode. See <code>imp/docs/INSTALL</code> for more information.',
+        'fatal' => false,
     )
 );
 
@@ -99,15 +118,15 @@ $file_list = array(
     'config/servers.php' => null
 );
 
-/* PEAR modules. */
+/* PEAR/PECL modules. */
 $pear_list = array(
-    'HTTP_Request' => array(
-        'path' => 'HTTP/Request.php',
-        'error' => 'The HTML composition mode requires HTTP_Request.'
-    ),
     'Auth_SASL' => array(
         'path' => 'Auth/SASL.php',
         'error' => 'If your IMAP server uses CRAM-MD5 or DIGEST-MD5 authentication, this module is required.'
+    ),
+    'HTTP_Request' => array(
+        'path' => 'HTTP/Request.php',
+        'error' => 'The HTML composition mode requires HTTP_Request.'
     )
 );
 
@@ -148,7 +167,7 @@ $user = isset($_POST['user']) ? $_POST['user'] : '';
 $passwd = isset($_POST['passwd']) ? $_POST['passwd'] : '';
 $type = isset($_POST['server_type']) ? $_POST['server_type'] : '';
 
-if (!empty($server) && !empty($user) && !empty($passwd) && !empty($type)) {
+if (!empty($server) && strlen($user) && strlen($passwd) && !empty($type)) {
     if ($type == 'pop') {
         $conn = array(
             'pop3/notls' => 110,
@@ -177,10 +196,10 @@ if (!empty($server) && !empty($user) && !empty($passwd) && !empty($type)) {
         if ($mbox) {
             $minfo = @imap_mailboxmsginfo($mbox);
             if ($minfo) {
-                echo '<font color="green">SUCCESS</font> - INBOX has ', $minfo->Nmsgs, ' messages (' . $minfo->Unread, ' new ', $minfo->Recent, ' recent)';
+                echo '<span style="color:green">SUCCESS</span> - INBOX has ', $minfo->Nmsgs, ' messages (' . $minfo->Unread, ' new ', $minfo->Recent, ' recent)';
                 $success[] = array('server' => $server, 'protocol' => $key, 'port' => $server_port);
             } else {
-                echo '<font color="red">ERROR</font> - The server returned the following error message:' . "\n" . '<pre>';
+                echo '<span style="color:red">ERROR</span> - The server returned the following error message:' . "\n" . '<pre>';
                 foreach (imap_errors() as $error) {
                     echo wordwrap($error);
                 }
@@ -188,7 +207,7 @@ if (!empty($server) && !empty($user) && !empty($passwd) && !empty($type)) {
             }
             @imap_close($mbox);
         } else {
-            echo '<font color="red">ERROR</font> - The server returned the following error message:' . "\n" . '<pre>';
+            echo '<span style="color:red">ERROR</span> - The server returned the following error message:' . "\n" . '<pre>';
             foreach (imap_errors() as $error) {
                 echo wordwrap($error);
             }
@@ -212,15 +231,15 @@ if (!empty($server) && !empty($user) && !empty($passwd) && !empty($type)) {
             echo "<strong>The following IMAP server information was discovered from the remote server:</strong>\n";
             $config = reset($success);
             require_once './lib/IMAP/Client.php';
-            $imapclient = &new IMP_IMAPClient($config['server'], $config['port'], $config['protocol']);
+            $imapclient = new IMP_IMAPClient($config['server'], $config['port'], $config['protocol']);
             $use_tls = $imapclient->useTLS();
             if (!is_a($use_tls, 'PEAR_Error')) {
                 $res = $imapclient->login($user, $passwd);
             }
             if (isset($res) && !is_a($res, 'PEAR_Error')) {
                 echo "<blockquote><em>Namespace Information</em><blockquote><pre>";
-                $namespace = $imapclient->namespace();
-                if (is_array($namespace)) {
+                $namespace = $imapclient->getNamespace();
+                if (!is_a($namespace, 'PEAR_Error')) {
                     foreach ($namespace as $val) {
                         echo "NAMESPACE: \"" . $val['name'] . "\"\n";
                         echo "DELIMITER: " . $val['delimiter'] . "\n";
@@ -231,8 +250,8 @@ if (!empty($server) && !empty($user) && !empty($passwd) && !empty($type)) {
                 }
                 echo "</pre></blockquote></blockquote>\n";
 
-                echo "<blockquote><em>IMAP CHILDREN support:</em><blockquote><pre>";
-                echo ($imapclient->queryCapability('CHILDREN')) ? 'SUPPORTED' : 'Not supported';
+                echo "<blockquote><em>IMAP server capabilities:</em><blockquote><pre>";
+                print_r($imapclient->queryCapability(null));
                 echo "</pre></blockquote></blockquote>\n";
 
                 echo "<blockquote><em>IMAP Charset Search Support:</em><blockquote><pre>";
@@ -254,11 +273,11 @@ if (!empty($server) && !empty($user) && !empty($passwd) && !empty($type)) {
     ?>
 <form name="form1" method="post" action="test.php">
 <table>
-<tr><td align="right">Server:</td><td><input type="text" name="server" /></td></tr>
-<tr><td align="right">Port:</td><td><input type="text" name="port" /></td><td>(If non-standard port; leave blank to auto-detect using standard ports)</td></tr>
-<tr><td align="right">User:</td><td><input type="text" name="user" /></td></tr>
-<tr><td align="right">Password:</td><td><input type="password" name="passwd" /></td></tr>
-<tr><td align="right">Server Type:</td><td><select name="server_type"><option value="imap">IMAP</option><option value="pop">POP</option></select></td></tr>
+<tr><td align="right"><label for="server">Server:</label></td><td><input type="text" id="server" name="server" /></td></tr>
+<tr><td align="right"><label for="port">Port:</label></td><td><input type="text" id="port" name="port" /></td><td>(If non-standard port; leave blank to auto-detect using standard ports)</td></tr>
+<tr><td align="right"><label for="user">User:</label></td><td><input type="text" id="user" name="user" /></td></tr>
+<tr><td align="right"><label for="passwd">Password:</label></td><td><input type="password" id="passwd" name="passwd" /></td></tr>
+<tr><td align="right"><label for="server_type">Server Type:</label></td><td><select id="server_type" name="server_type"><option value="imap">IMAP</option><option value="pop">POP</option></select></td></tr>
 <tr><td></td><td><input type="submit" name="f_submit" value="Submit" /><input type="reset" name="f_reset" value="Reset" /></td></tr>
 </table>
 </form>
